//+------------------------------------------------------------------+
//|                                                                  |
//| idea for zero lag hull moving average                            |
//| originaly by spikoloco                                           }
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  DeepSkyBlue
#property indicator_color2  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern int    Length          = 20;
extern int    Price           = PRICE_CLOSE;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double hma[];
double hmahUp[];
double hmahDn[];
double hma1[];
double hma2[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,hmahUp); SetIndexStyle(0,DRAW_HISTOGRAM);SetIndexLabel(0,"UP");
      SetIndexBuffer(1,hmahDn); SetIndexStyle(1,DRAW_HISTOGRAM);SetIndexLabel(1,"DWN");
      SetIndexBuffer(2,hma);
      SetIndexBuffer(3,hma1); 
      SetIndexBuffer(4,hma2); 
      SetIndexBuffer(5,trend); 

      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
   //
   //
   //
   //
   //
         
   IndicatorShortName(timeFrameToString(timeFrame)+" zero lag HMA ("+Length+")");         
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { trend[0] = limit+1; return(0); }
         
   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         hma1[i] = iHma(iMA(NULL,0,1,0,MODE_SMA,Price,i),Length,i,0);
         hma2[i] = iHma(hma1[i]                         ,Length,i,2);
         hma[i]  = 2.0*hma1[i]-hma2[i];

         //
         //
         //
         //
         //
         
         hmahUp[i] = EMPTY_VALUE;
         hmahDn[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
            if (hma[i] > hma[i+1]) trend[i] =   1;
            if (hma[i] < hma[i+1]) trend[i] =  -1;
            if (trend[i] ==  1) hmahUp[i] = 1;
            if (trend[i] == -1) hmahDn[i] = 1;
      }
      manageAlerts();
      return(0);
   }      

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         trend[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,Price,5,y);
         hmahUp[i] = EMPTY_VALUE;
         hmahDn[i] = EMPTY_VALUE;
            if (trend[i] ==  1) hmahUp[i] = 1;
            if (trend[i] == -1) hmahDn[i] = 1;
   }
   
   //
   //
   //
   //
   //
   
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][4];

//
//
//
//
//

double iHma(double price, int period, int i, int s=0)
{
   if (ArrayRange(wrk,0)!=Bars) ArrayResize(wrk,Bars);
   
   //
   //
   //
   //
   //
   
      int HMAPeriod  = MathMax(2,period);
      int HalfPeriod = MathFloor(HMAPeriod/2);
      int HullPeriod = MathFloor(MathSqrt(HMAPeriod));

      //
      //
      //
      //
      //
         
      int r = Bars-i-1;  
         wrk[r][s  ] = price;
         wrk[r][s+1] = 2.0*iLwma(s,HalfPeriod,r)-iLwma(s,HMAPeriod,r);
      return (iLwma(s+1,HullPeriod,r));
}

//+----------------------------------------------------------------------------------+
//|                                                                                  |
//+----------------------------------------------------------------------------------+
//
//
//
//
//

double iLwma(int forBuffer, int period, int r)
{
   double weight = 0;
   double sum    = 0;
   int    i,k;
   
   if (r>=period)
   {
      for (i=0, k=period; i<period; i++,k--)
      {
            weight += k; sum += wrk[r-i][forBuffer]*k;
      }
      if (weight !=0)
            return(sum/weight);
      else  return(0.0);
    }
    else return(wrk[r][forBuffer]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ZL hull trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"ZL hull"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}